/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import org.apache.log4j.Logger;

import com.eviware.soapui.impl.wsdl.submit.RequestTransport;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.iface.Response;
import com.eviware.soapui.model.iface.Submit;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.model.iface.SubmitListener;

/**
 * Submit implementation for submitting a WsdlRequest
 * 
 * @author Ole.Matzura
 */

public final class WsdlSubmit implements Runnable, Submit
{
	private final static Logger logger = Logger.getLogger(WsdlSubmit.class);
	private WsdlRequest wsdlRequest;
	private SubmitListener[] listeners;
	private Status status;
	private Exception error;
	private Response response;
	private volatile Future future;
	private SubmitContext submitContext;
	private final static ExecutorService threadPool = Executors.newCachedThreadPool();
	private RequestTransport transport;

	public WsdlSubmit(WsdlRequest wsdlRequest, SubmitListener[] listeners, RequestTransport transport)
	{
		this.wsdlRequest = wsdlRequest;
		this.transport = transport;
		
		this.listeners = new SubmitListener[listeners.length];
		for( int c = 0; c < listeners.length; c++ )
			this.listeners[c] = listeners[c];
		
		error = null;
		status = Status.INITIALIZED;
		future = null;
	}

	public void submitRequest(SubmitContext submitContext, boolean async )
	{
		this.submitContext = submitContext;

		if( async && future != null )
			throw new RuntimeException( "Submit already running" );
		
		if( async )
			future = threadPool.submit(this);
		else
			run();
	}

	public void cancel()
	{
		if (status == Status.CANCELED)
			return;

		logger.info("Canceling request..");
		if (status == Status.RUNNING )
		{
			transport.abortRequest( submitContext );
		}

		status = Status.CANCELED;

		for (int i = 0; i < listeners.length; i++)
		{
			listeners[i].afterSubmit(this, submitContext);
		}
	}

	public void run()
	{
		try
		{
			submitContext.setProperty( RequestTransport.REQUEST_TRANSPORT, transport );
	      submitContext.setProperty( RequestTransport.WSDL_REQUEST, wsdlRequest );
			
			for (int i = 0; i < listeners.length; i++)
			{
				if (!listeners[i].beforeSubmit(this, submitContext))
				{
					status = Status.CANCELED;
					System.err.println("listener cancelled submit..");
					return;
				}
			}

			status = Status.RUNNING;
			response = transport.sendRequest(submitContext, wsdlRequest);
			
			if (status != Status.CANCELED)
			{
				status = Status.FINISHED;
			}

			if( response.getTimeTaken() == 0 )
			{
				logger.warn( "Request took 0 in thread " + Thread.currentThread().getId() + 
						", response length = " + response.getContentLength() );
			}
		}
		catch (Exception e1)
		{
			error = e1;
			status = Status.ERROR;
			logger.error("Exception in request: " + e1);
			e1.printStackTrace();
		}
		finally
		{
			if (status != Status.CANCELED)
			{
				for (int i = 0; i < listeners.length; i++)
				{
					listeners[i].afterSubmit(this, submitContext);
				}
			}
		}
	}

	public Request getRequest()
	{
		return wsdlRequest;
	}

	public Status getStatus()
	{
		return status;
	}

	public Exception getError()
	{
		return error;
	}

	public synchronized Status waitUntilFinished()
	{
		if (future != null)
		{
			if (!future.isDone())
			{
				try
				{
					future.get();
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
			}
		}
		else
			throw new RuntimeException("cannot wait on null future");

		return getStatus();
	}

	public Response getResponse()
	{
		return response;
	}
}
