/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.testsuite;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;

import com.eviware.soapui.impl.wsdl.WsdlTestSuite;
import com.eviware.soapui.impl.wsdl.panels.support.ProgressBarAdapter;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.model.support.TestSuiteListenerAdapter;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionSupport;

/**
 * A panel showing a scrollable list of TestCases in a TestSuite.
 * 
 * @author Ole.Matzura
 */

public class JTestCaseList extends JPanel
{
	private Map<TestCase,TestCaseListPanel> panels = new HashMap<TestCase,TestCaseListPanel>();
	private JPopupMenu popup;
	private final WsdlTestSuite testSuite;
	private final InternalTestSuiteListener testSuiteListener = new InternalTestSuiteListener();

	public JTestCaseList(WsdlTestSuite testSuite)
	{
		this.testSuite = testSuite;
		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ));
		
		for( int c = 0; c < testSuite.getTestCaseCount(); c++ )
		{
			TestCaseListPanel testCaseListPanel = new TestCaseListPanel(testSuite.getTestCaseAt(c));
			panels.put( testSuite.getTestCaseAt( c ), testCaseListPanel );
			add( testCaseListPanel );
		}
		
		add( Box.createVerticalGlue() );
		setBackground( Color.WHITE );
		
		testSuite.addTestSuiteListener( testSuiteListener );
	}
	
	@Override
	public void addNotify()
	{
		super.addNotify();
		testSuite.addTestSuiteListener( testSuiteListener );
	}

	@Override
	public void removeNotify()
	{
		super.removeNotify();
		testSuite.removeTestSuiteListener( testSuiteListener );
	}

	private final class InternalTestSuiteListener extends TestSuiteListenerAdapter
	{
		public void testCaseAdded(TestCase testCase)
		{
			TestCaseListPanel testCaseListPanel = new TestCaseListPanel(( WsdlTestCase ) testCase);
			panels.put( testCase, testCaseListPanel );
			add( testCaseListPanel, getComponentCount()-1 );
			revalidate();
			repaint();
		}

		public void testCaseRemoved(TestCase testCase)
		{
			TestCaseListPanel testCaseListPanel = panels.get( testCase );
			if( testCaseListPanel != null )
			{
				remove( testCaseListPanel );
				panels.remove( testCase );
				revalidate();
				repaint();
			}
		}
	}
	
	private final static class TestCaseListPanel extends JPanel
	{
		private final TestCase testCase;
		private JProgressBar progressBar;
		private JLabel label;
		private ProgressBarAdapter progressBarAdapter;
		private boolean selected;
		private TestCasePropertyChangeListener testCasePropertyChangeListener;

		public TestCaseListPanel( WsdlTestCase testCase )
		{
			super( new BorderLayout() );
			this.testCase = testCase;
			
			progressBar = new JProgressBar( 0, 100 );
			JPanel progressPanel = UISupport.createProgressBarPanel( progressBar, 5, false );
			
		   progressBar.setMinimumSize( new Dimension( 0, 10 ));
		   progressBar.setBackground( Color.WHITE );
		   progressBar.setInheritsPopupMenu( true );
		   progressBarAdapter = new ProgressBarAdapter( progressBar, testCase );
		   
			label = new JLabel( "TestCase: " + testCase.getName() );
			label.setBorder( BorderFactory.createEmptyBorder(5, 5, 5, 5));
		   label.setInheritsPopupMenu( true );

			add( progressPanel, BorderLayout.CENTER );
			add( label, BorderLayout.NORTH );
			
			testCasePropertyChangeListener = new TestCasePropertyChangeListener();
			testCase.addPropertyChangeListener( TestCase.NAME_PROPERTY, testCasePropertyChangeListener );
			
			setComponentPopupMenu( ActionSupport.buildPopup( testCase.getActions() ) );
			
			addMouseListener( new MouseAdapter() {
				public void mouseClicked(MouseEvent e)
				{
					if (e.getClickCount() < 2)
					{
						setSelected( !selected );
						return;
					}
					
					UISupport.selectAndShow( TestCaseListPanel.this.testCase );
				}
			} );
			
			setSelected( false );
		}
		
		public void removeNotify()
		{
			super.removeNotify();
			testCase.removePropertyChangeListener( testCasePropertyChangeListener );
		}

		public Dimension getMaximumSize() 
		{
		    Dimension size = super.getMaximumSize();
		    size.height = 50;
		    return size;
		}
		
		public void setSelected( boolean selected )
		{
			this.selected = selected;
			
			if( selected )
			{
				setBackground( Color.YELLOW.brighter().brighter() );
				setBorder( BorderFactory.createLineBorder( Color.GRAY ));
			}
			else
			{
				setBackground( Color.WHITE );
				setBorder( BorderFactory.createLineBorder( Color.WHITE ));
			}
		}

		public boolean isSelected()
		{
			return selected;
		}

		private final class TestCasePropertyChangeListener implements PropertyChangeListener
		{
			public void propertyChange(PropertyChangeEvent evt)
			{
				label.setText( "TestCase: " + TestCaseListPanel.this.testCase.getName() );
			}
		}
	}
	
	public int[] getSelectedIndices()
	{
		int cnt = 0;
		for( TestCaseListPanel panel : panels.values() )
		{
			if( panel.isSelected() ) cnt++;
		}
			
		int [] result = new int[cnt];
		cnt = 0;
		
		for( int c = 0; c < getComponentCount(); c++ )
		{
			Component comp = getComponent( c );
			if( comp instanceof TestCaseListPanel && ((TestCaseListPanel)comp).isSelected() )
			{
				result[cnt] = c;
				cnt++;
			}
		}
		
		return result;
	}

}
