/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.support.wsdl;

import java.io.File;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.httpclient.HttpState;
import org.apache.xmlbeans.SimpleValue;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;

import com.eviware.soapui.config.DefinitionCacheConfig;
import com.eviware.soapui.config.DefintionPartConfig;
import com.eviware.soapui.impl.wsdl.support.Constants;
import com.eviware.soapui.support.Tools;

public class CachedWsdlLoader extends WsdlLoader 
{
	private HttpState state;
	private final DefinitionCacheConfig config;
	
	public CachedWsdlLoader(DefinitionCacheConfig config)
	{
		super( config.getRootPart() );
		this.config = config;
	}

	public CachedWsdlLoader(String wsdlUrl) throws Exception
	{
		this( WsdlLoader.cacheWsdl( new UrlWsdlLoader( wsdlUrl )));
	}

	public InputStream load( String url ) throws Exception
	{
		XmlObject xmlObject = loadXmlObject( url, null );
		return xmlObject == null ? null : xmlObject.newInputStream();
	}

	public XmlObject loadXmlObject(String url, XmlOptions options ) throws Exception
	{
		List<DefintionPartConfig> partList = config.getPartList();
		for( DefintionPartConfig part : partList )
		{
			if( part.getUrl().equalsIgnoreCase( url ))
			{
				return XmlObject.Factory.parse( part.getContent().toString(), new XmlOptions().setLoadLineNumbers() );
			}
		}
		
		return null;
	}

	public boolean abort()
	{
		return false;
	}
	
	public boolean isAborted()
	{
		return false;
	}

	public String saveDefinition( String folderName ) throws Exception
	{
		File outFolder = new File( folderName );
		if( !outFolder.exists() && !outFolder.mkdirs() )
			throw new Exception( "Failed to create directory [" + folderName + "]" );
		
		Map<String,String> urlToFileMap = new HashMap<String,String>(); 
		
		setFilenameForUrl( config.getRootPart(), Constants.WSDL11_NS, urlToFileMap );
		
		List<DefintionPartConfig> partList = config.getPartList();
		for( DefintionPartConfig part : partList )
		{
			setFilenameForUrl( part.getUrl(), part.getType(), urlToFileMap );
		}
		
		for( DefintionPartConfig part : partList )
		{
			XmlObject obj = XmlObject.Factory.parse( part.getContent().toString() );
			replaceImportsAndIncludes( obj, urlToFileMap, part.getUrl() );
			obj.save( new File( outFolder, urlToFileMap.get( part.getUrl() )));
		}
		
		return folderName + File.separatorChar + urlToFileMap.get( config.getRootPart() );
	}
	
	private void setFilenameForUrl( String fileUrl, String type, Map<String,String> urlToFileMap ) throws MalformedURLException
	{
		URL url = new URL( fileUrl );
		String path = url.getPath();
		
		int ix = path.lastIndexOf( '/' );
		String fileName = ix == -1 ? path : path.substring( ix+1 );
		
		ix = fileName.lastIndexOf( '.');
		if( ix != -1 )
			fileName = fileName.substring( 0 , ix );
			
		if( type.equals( Constants.WSDL11_NS ))
			fileName += ".wsdl";
		else if( type.equals( Constants.XSD_NS ))
			fileName += ".xsd";
		else
			fileName += ".xml";
		
		while( urlToFileMap.containsValue( fileName ))
		{
			ix = fileName.lastIndexOf( '.');
			fileName = fileName.substring( 0, ix ) + "_" + fileName.substring( ix );
		}
		
		urlToFileMap.put( fileUrl, fileName );
	}

	private void replaceImportsAndIncludes(XmlObject xmlObject, Map<String, String> urlToFileMap, String baseUrl ) throws Exception
	{
		XmlObject[] wsdlImports = xmlObject
			.selectPath("declare namespace s='http://schemas.xmlsoap.org/wsdl/' .//s:import/@location");
		
		for (int i = 0; i < wsdlImports.length; i++)
		{
			SimpleValue wsdlImport = ((SimpleValue) wsdlImports[i]);
			replaceLocation(urlToFileMap, baseUrl, wsdlImport);
		}
		
		XmlObject[] schemaImports = xmlObject
			.selectPath("declare namespace s='http://www.w3.org/2001/XMLSchema' .//s:import/@schemaLocation");

		for (int i = 0; i < schemaImports.length; i++)
		{
			SimpleValue schemaImport = ((SimpleValue) schemaImports[i]);
			replaceLocation(urlToFileMap, baseUrl, schemaImport);
		}

		XmlObject[] schemaIncludes = xmlObject
			.selectPath("declare namespace s='http://www.w3.org/2001/XMLSchema' .//s:include/@schemaLocation");
		for (int i = 0; i < schemaIncludes.length; i++)
		{
			SimpleValue schemaInclude = ((SimpleValue) schemaIncludes[i]);
			replaceLocation(urlToFileMap, baseUrl, schemaInclude);
		}
	}

	private void replaceLocation(Map<String, String> urlToFileMap, String baseUrl, SimpleValue wsdlImport) throws Exception
	{
		String location = wsdlImport.getStringValue();
		if (location != null)
		{
			if (location.startsWith("file:")
					|| location.indexOf("://") > 0)
			{
				String newLocation = urlToFileMap.get( location );
				if( newLocation != null )
					wsdlImport.setStringValue( newLocation);
				else
					throw new Exception( "Missing local file for [" + newLocation + "]" );
			}
			else
			{
				String loc = Tools.joinRelativeUrl(baseUrl, location);
				String newLocation = urlToFileMap.get( loc);
				if( newLocation != null )
					wsdlImport.setStringValue( newLocation);
				else
					throw new Exception( "Missing local file for [" + loc + "]" );
			}
		}
	}

	public void close()
	{
	}
}