/*
 *  soapUI, copyright (C) 2006 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import groovy.lang.Binding;
import groovy.lang.GroovyShell;
import groovy.lang.Script;

import javax.swing.ImageIcon;

import org.apache.log4j.Logger;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.teststeps.actions.CloneTestStepAction;
import com.eviware.soapui.model.support.DefaultTestStepProperty;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStepResult;
import com.eviware.soapui.model.testsuite.TestRunner.Status;
import com.eviware.soapui.model.testsuite.TestStepResult.TestStepStatus;
import com.eviware.soapui.monitor.TestMonitor;
import com.eviware.soapui.support.ScriptingSupport;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionSupport;
import com.eviware.soapui.support.xml.XmlObjectConfigurationBuilder;
import com.eviware.soapui.support.xml.XmlObjectConfigurationReader;

public class WsdlGroovyScriptTestStep extends WsdlTestStep
{
	private final static Logger logger = Logger.getLogger( "groovy.log" );
	private String scriptText = "";
	private Object scriptResult;
	private ImageIcon failedIcon;
	private ImageIcon okIcon;
	private Binding binding;
	private GroovyShell shell;
	private Script script;
	
	public WsdlGroovyScriptTestStep(WsdlTestCase testCase, TestStepConfig config)
	{
		super(testCase, config, false);
		
		addAction( ActionSupport.SEPARATOR_ACTION );
		addAction( new CloneTestStepAction( this, "Groovy Script" ) );
		addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new ShowOnlineHelpAction( HelpUrls.GROOVYSTEP_HELP_URL ));
		
		okIcon = UISupport.createImageIcon("/groovy_script.gif");
		setIcon( okIcon );
		failedIcon = UISupport.createImageIcon( "/groovy_script_failed.gif" );

		if( config.getConfig() == null )
		{
			saveScript( config );
		}
		else
		{
			readConfig( config );
		}
		
		addProperty( new DefaultTestStepProperty( "result", true, 
				new DefaultTestStepProperty.PropertyHandlerAdapter() {

			public String getValue()
			{
				return scriptResult == null ? null : scriptResult.toString();
			}
		}, this ));
		
		binding = new Binding();
		shell = ScriptingSupport.createGroovyShell( binding);
	}
	
	public Logger getLogger()
	{
		SoapUI.ensureGroovyLog();
		return logger;
	}
	
	private void readConfig(TestStepConfig config)
	{
		XmlObjectConfigurationReader reader = new XmlObjectConfigurationReader( config.getConfig() );
		scriptText = reader.readString( "script", "" );
	}
	
	private void saveScript(TestStepConfig config)
	{
		XmlObjectConfigurationBuilder builder = new XmlObjectConfigurationBuilder();
		builder.add( "script", scriptText );
		config.setConfig( builder.finish() );
	}
	
	public void resetConfigOnMove(TestStepConfig config)
	{
		super.resetConfigOnMove( config );
		readConfig( config );
	}

	public TestStepResult run(TestRunner testRunner, TestRunContext context)
	{
		TestMonitor testMonitor = SoapUI.getTestMonitor();
		boolean hasRunningLoadTest = testMonitor == null ? false : testMonitor.hasRunningLoadTest( getTestCase() );
		
		SoapUI.ensureGroovyLog();
		
		WsdlTestStepResult result = new WsdlTestStepResult( this );
		Logger log = (Logger) context.getProperty( "log" );
		if( log == null )
			log = logger;
		
		try
		{
			if( scriptText.trim().length() > 0 )
			synchronized( this )
			{
				binding.setVariable("context", context);
				binding.setVariable("testRunner", testRunner);
				binding.setVariable( "log", log);
				
				if( script == null )
				{
					script = shell.parse( scriptText );
					script.setBinding( binding );
				}
				
				result.setTimeStamp( System.currentTimeMillis() );
				result.startTimer();
				scriptResult = script.run();
				result.stopTimer();
				
				if( scriptResult != null )
					result.addMessage( "Script-result: " + scriptResult.toString() );
			}
			
			// testRunner status may have been changed by script..
			Status testRunnerStatus = testRunner.getStatus();
			if( testRunnerStatus == Status.FAILED )
				result.setStatus( TestStepStatus.FAILED );
			else if( testRunnerStatus == Status.CANCELED )
				result.setStatus( TestStepStatus.CANCELED );
			else
				result.setStatus( TestStepStatus.OK );
		}
		catch (Throwable e)
		{
			log.error( e );
			result.stopTimer();
			result.addMessage( e.getMessage() );
			result.setError( e );
			result.setStatus( TestStepStatus.FAILED );
		}		
		finally
		{
			if( !hasRunningLoadTest )
				setIcon( testRunner.getStatus() == Status.FAILED ? failedIcon : okIcon );
		}
		
		return result;
	}

	public String getScript()
	{
		return scriptText;
	}
	
	public void setScript( String scriptText )
	{
		if( scriptText.equals( this.scriptText ))
			return;
		
		this.scriptText = scriptText;
		script = null;
		saveScript( getConfig() );
	}
}

